/**
 * @file FdSerial.cpp
 *
 * Full Duplex Serial class.
 *
 * Based heavily on FdSerial.c by Steve Denson
 *
 * Copyright (c) 2008, Steve Denson
 * See end of file for terms of use.
 */
 
#include <propeller.h>
#include "FdSerial.h"

/* maximum positive and negative values */
#define POSX    0x7fffffff
#define NEGX    0x80000000

/**
 * start initializes and starts native assembly driver in a cog.
 * @param rxpin is pin number for receive input
 * @param txpin is pin number for transmit output
 * @param mode is interface mode. see header FdSerial::MODE_...
 * @param baudrate is frequency of bits ... 115200, 57600, etc...
 * @returns non-zero on success
 */
int FdSerial::start(int rxpin, int txpin, int modex, int baudrate)
{
    use_cog_driver(FdSerialDriver);
    
    rx_pin  = rxpin;                  // receive pin
    tx_pin  = txpin;                  // transmit pin
    mode    = modex;                  // interface mode
    ticks   = _clkfreq / baudrate;    // baud
    buffptr = (int)&rxbuff[0];
    cogId = load_cog_driver(FdSerialDriver, this) + 1;

    return cogId;
}

/**
 * stop stops the cog running the native assembly driver 
 */
void FdSerial::stop()
{
    if(cogId > 0) {
        cogstop(cogId - 1);
        cogId = 0;
    }
}
/**
 * rxflush empties the receive queue 
 */
void FdSerial::rxflush()
{
    // clear out queue by receiving all available 
    while(rxcheck() >= 0)
      ;
}

/**
 * Gets a byte from the receive queue if available
 * Function does not block. We move rxtail after getting char.
 * @returns receive byte 0 to 0xff or -1 if none available 
 */
int FdSerial::rxcheck()
{
    int rc = -1;
    if(rx_tail != rx_head) {
        rc = rxbuff[rx_tail];
        rx_tail = (rx_tail+1) & BUF_MASK;
    }
    return rc;
}

/**
 * Wait for a byte from the receive queue. blocks until something is ready
 * @returns received byte 
 */
int FdSerial::rx()
{
  int rc = rxcheck();
  while(rc < 0)
    rc = rxcheck();
  return rc;
}

/**
 * putbyte sends a byte on the transmit queue.
 * @param txbyte is byte to send. 
 */
int FdSerial::tx(int txbyte)
{
    while(tx_tail == ((tx_head+1) & BUF_MASK)) // wait for space in queue
        ;

    txbuff[tx_head] = txbyte;
    tx_head = (tx_head+1) & BUF_MASK;
    return txbyte;
}

void FdSerial::str(const char *stringptr)
{

// Send string                    

  while (*stringptr)
    tx(*stringptr++);
}
    
void FdSerial::dec(int value)
{
#if 0
  int i, x, cnt;

// Print a decimal number

  x = (value == NEGX);                                                          // Check for max negative
  if (value < 0) {
    value := ||(value+x)                                                        // If negative, make positive; adjust for max negative
    tx("-")                                                                     // and output sign
  }

  i = 1000000000;                                                               // Initialize divisor

  for (cnt = 10; --cnt >= 0; ) {                                                // Loop for 10 digits
    if (value >= i) {                                                              
      tx(value / i + '0' + x * (i == 1));                                       // If non-zero digit, output digit; adjust for max negative
      value %= i;                                                               // and digit from value
      result~~;                                                                 // flag non-zero found
    }
    else if (result || i == 1)
      tx('0');                                                                  // If zero digit (or only digit) output it
    i /= 10;  
  }                                                                             // Update divisor
#endif
}

void FdSerial::hex(int value, int digits)
{
  
// Print a hexadecimal number

  value <<= (8 - digits) << 2;
  while (--digits >= 0) {
    int digit = (value >> 28) & 0xf;
    tx(digit >= 10 ? digit - 10 + 'A' : digit + '0');
    value <<= 4;
  }
}

void FdSerial::bin(int value, int digits)
{
  
// Print a binary number

  value <<= 32 - digits;
  while (--digits >= 0) {
    int digit = (value >> 31) & 0x1;
    tx(digit + '0');
    value <<= 1;
  }
}

FdSerial &FdSerial::operator<<(int txbyte)
{
    tx(txbyte);
    return *this;
}

FdSerial &FdSerial::operator<<(const char *p)
{
    while (*p)
        tx(*p++);
    return *this;
}

/**
 * drain waits for all bytes to be sent
 */
void FdSerial::drain()
{
  unsigned int waitcycles;
  while(tx_tail != tx_head)
    ;

  // wait for character to be transmitted
  // strictly speaking we only need to wait 10*ticks,
  // but give ourselves a bit of margin here
  waitcycles = CNT + (ticks<<4);
  waitcnt(waitcycles);
}

/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/
