/**
 * @file FdSerial.h
 * Full Duplex Serial adapter API definition.
 *
 * Copyright (c) 2008, Steve Denson
 * See end of file for terms of use.
 */
#ifndef __FdSerial__
#define __FdSerial__

/**
 * Defines FdSerial class
 * 9 contiguous longs + buffers + COG ID
 * These buffers must be contiguous. Their size must match the asm expectation.
 * Asm also expects the address of txbuff to be after rxbuff.
 */
class FdSerial
{
public:

    /**
     * Defines mode bits
     *   mode bit 0 = invert rx
     *   mode bit 1 = invert tx
     *   mode bit 2 = open-drain/source tx
     */

    static const int MODE_NONE = 0;
    static const int MODE_INVERT_RX = 1;
    static const int MODE_INVERT_TX = 2;
    static const int MODE_OPENDRAIN_TX = 4;
    
    FdSerial() : cogId(0)
    {
    }
    
    FdSerial(int rxpin, int txpin, int mode, int baudrate) : cogId(0)
    {
        start(rxpin, txpin, mode, baudrate);
    }
    
    ~FdSerial()
    {
        stop();
    }
    
    /**
     * start initializes and starts native assembly driver in a cog. 
     * @param rxpin is pin number for receive input
     * @param txpin is pin number for transmit output
     * @param mode is interface mode
     * @param baudrate is frequency of bits ... 115200, 57600, etc...
     * @returns non-zero on success
     */
    int start(int rxpin, int txpin, int mode, int baudrate);
    /**
     * stop stops the cog running the native assembly driver 
     */
    void stop();
    
    /**
     * rxflush empties the receive queue 
     */
    void rxflush();
    
    /**
     * rxcheck gets a byte from the receive queue if available
     * function does not block.
     * @returns receive byte 0 to 0xff or -1 if none available 
     */
    int rxcheck();
    
    /**
     * rxtime gets a byte from the receive queue if available by timeout
     * function blocks if no recieve for ms timeout.
     * @param ms is number of milliseconds to wait for a char
     * @returns receive byte 0 to 0xff or -1 if none available 
     */
    int rxtime(int ms);
    
    /**
     * rx waits for a byte from the receive queue. blocks until somehting is ready
     * @returns received byte 
     */
    int rx();
    
    /**
     * tx sends a byte on the transmit queue.
     * @param txbyte is byte to send. 
     * @returns waits for and returns received byte if mode is 8 
     */
    int tx(int txbyte);
    
    void str(const char *stringptr);
    void dec(int value);
    void hex(int value, int digits);
    void bin(int value, int digits);

    /**
     * << sends a byte on the transmit queue
     *
     */
    FdSerial &operator<<(int txbyte);

    /**
     * << sends each byte in a string on the transmit queue
     *
     */
    FdSerial &operator<<(const char *p);

    /**
     * drain waits for all bytes to be sent
     */
    void drain();

private:

    /**
     * Defines buffer length. hard coded in asm driver ... s/b bigger
     */
    static const int BUF_SIZE = 64;
    static const int BUF_MASK = BUF_SIZE - 1;

    volatile int rx_head;   // receive queue head
    volatile int rx_tail;   // receive queue tail
    volatile int tx_head;   // transmit queue head
    volatile int tx_tail;   // transmit queue tail
    int rx_pin;             // receive pin
    int tx_pin;             // transmit pin
    int mode;               // interface mode
    int ticks;              // clkfreq / baud
    int buffptr;            // pointer to rx buffer
    char rxbuff[BUF_SIZE];  // receive buffer
    char txbuff[BUF_SIZE];  // transmit buffer
    int cogId;              // cog flag/id
};

#endif 

/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/

