// =================================================================================================
//
//   File....... jm_sircs_tx.cpp
//   Purpose.... SIRCS-compatible transmitter
//               -- works with active-high and active-low connections
//   Author..... Jon "JonnyMac" McPhalen
//               Copyright (c) 2009-13 Jon McPhalen
//               -- see below for terms of use
//   E-mail..... jon@jonmcphalen.com
//   Started.... 
//   Updated.... 01 MAY 2013
//
// =================================================================================================

/*

    Typical connections:

              IR
      3.3v ──────┐
                    │
    ir pin ──────┘
               33



    Parallax QuickStart HID Board uses active-high output to IR LED 

          33  IR
    P9 ───────┐
                   
                    

    Protocol Reference:
    -- http://www.sbprojects.com/knowledge/ir/sirc.php
              
*/

#include <propeller.h>
#include "jm_SircsTx.h"

#define MIN(a, b) ((a) < (b) ? (a) : (b))
#define MAX(a, b) ((a) > (b) ? (a) : (b))

int jm_SircsTx::start(int pin, int hz)
{

// Starts SIRCS cog with 45ms frame timing (SIRCS standard)

  return startx(pin, hz, 45);
}
  
int jm_SircsTx::startx(int pin, int hz, int framems)
{
    use_cog_driver(jm_sircs_tx);

// Starts SIRCS cog with user-defined frame timing
// -- pin is modulation output
// -- hz is modulation frequency (e.g., 40_000)
// -- framems is frame timing in milliseconds
//    * warning: will truncate transmission if too short
//    * minimum 6ms + (bits * 1.8ms)

  stop();                                                       // stop if running
   
  irframes = pin;
  ircode   = (0x80000000 / (_clkfreq / hz)) << 1;               // setup for ctrx NCO mode
  irbits   = _clkfreq / 1000 * MAX(10, MIN(framems, 100));      // frame timing
  irstart  = _clkfreq / 1000000 * 2400;                         // ticks in 2.4ms
   
  cogId = load_cog_driver(jm_sircs_tx, this);

  return cogId;
}

void jm_SircsTx::stop()
{

// Stops IR cog if running

  if (cogId >= 0) {
    cogstop(cogId);
    cogId = -1;
  }

  irframes = ircode = irbits = irstart = 0;                     // clear parameters
}    

void jm_SircsTx::tx(int code, int bits, int frames)
{

// Transmits SIRCS code
// -- code is the code to transmit
// -- bits is the bit count to transmit  (usually 12 or 20)
// -- frames is the number of transmitted frames (typically 3 to 5)

  if ((bits > 0) && (frames > 0))  {
    ircode   = code;
    irbits   = MAX(1, MIN(bits, 32));                           // limit bits
    irframes = MAX(1, MIN(frames, 5));                          // limit frames  
  }
}

int jm_SircsTx::busy()
{

// Reports returns true while code is transmitting

  return (irframes > 0);
}

/*

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
